/*
  LittleBits light sensitive MIDI music controller
  Alex P June 2022

  The Littlebits W6 Arduino Leonardo module can send MIDI signals via USB
  Updated from Light Sensitive MIDI Controller (Arduino) on pastebin.com
  Littlebit modules:
  main module is the W6 Arduino Leonardo module
  power snap + I3 button on input D0 for instrument selection
  power snap + I6 dimmer on input A0 for tempo selection
  P1 power + I13 light sensor on input A1 for pitch level selection
  O3 rgb LED on output D5 for vibrato ON/OFF indication
  O9 bargraph LED on output D9 for pitch (light) level indication

*/

//**********************LIBRARIES**************************************
#include "MIDIUSB.h"

//**********************VARIABLES**************************************
int lightPin = A1;
int potPin = A0;
int instrumentPin = 0;
int instrument;
int lightLED = 9;
int vibratoLED = 5;
int nn;
int minLight;
int maxLight;
int lightLevel;
int lightMap;
int potMap;
int toneMap;
int potLevel;
int potMem;
int select = 0;
int step = 0;
int InstrArray[16] = {0, 9, 10, 55, 68, 77, 86, 98, 99, 100, 102, 104, 106, 107, 112, 114};

void setup() {
  pinMode(5, OUTPUT);
  pinMode(9, OUTPUT);

  Serial.begin(9600);

  //**********************INITIAL LIGHT LEVEL DETECTION********************
  lightLevel = analogRead(lightPin);
  minLight = lightLevel - 20;
  maxLight = lightLevel;
}

void loop() {

  //**********************LIGHT LEVEL WHEN PLAYING*************************
  lightLevel = analogRead(lightPin);
  if (minLight > lightLevel) minLight = lightLevel;
  if (maxLight < lightLevel) maxLight = lightLevel;
  lightMap = map(lightLevel, minLight, maxLight, 0, 255);
  analogWrite(lightLED, lightMap);

  //**********************TEMPO LEVEL SETTING******************************
  potLevel = analogRead(potPin);
  potMap = map(potLevel, 0, 1023, 7, 255);

  //**********************TONE SETTING (PENTATONIC)************************
  toneMap = map(lightLevel, minLight, maxLight, 24, 96);
  nn = toneMap;
  while (nn >= 12) nn -= 12;
  // 0 C  1 C#  2 D  3 Eb  4 E  5 F  6 F#  7 G  8 Ab  9 A  10 Bb  11 B
  //   C    -     D    -     E    -    -     G          A
  if (nn == 1 || nn == 3 || nn == 5 || nn == 8 || nn == 10) toneMap--;
  else if (nn == 6 || nn == 11) toneMap++;

  //**********************VIBRATO ON/OFF SETTING***************************
  if (potMap > 250 && potMem == 0)
  {
    controlChange(1, 1, 127);
    digitalWrite(vibratoLED, HIGH);
    potMem = 1;
  }
  if (potMap < 20 && potMem == 1)
  {
    controlChange(1, 1, 0);
    digitalWrite(vibratoLED, LOW);
    potMem = 0;
  }

  //**********************INSTRUMENT SELECTION*****************************
  select = digitalRead(instrumentPin);
  if (select == HIGH)
  {
    instrument = InstrArray[step];
    step = step + 1;
    if (step > 15 ) {
      step = 0;
    }
    programChange(1, instrument);
    MidiUSB.flush();
    delay(50);
  }

  //**********************PLAY MIDI NOTES**********************************
  noteOn(1, toneMap, 127);
  delay(potMap );
  delay(random(50));
  noteOff(1, toneMap, 127);
  MidiUSB.flush();

  //**********************DEBUGGING VIA MONITOR****************************
  //Uncomment for debugging:
  //Serial.print("Tone mapped: ");
  //Serial.print(toneMap);
  //Serial.println();
  //Serial.print("Note delay: ");
  //Serial.print(potMap);
  //Serial.println();
  //Serial.print(instrument);
  //Serial.println();

}

//**********************MIDI SUBROUTINES*********************************
void noteOn(byte channel, byte pitch, byte velocity) {
  midiEventPacket_t noteOn = {0x09, 0x90 | channel - 1, pitch, velocity};
  MidiUSB.sendMIDI(noteOn);
}

void noteOff(byte channel, byte pitch, byte velocity) {
  midiEventPacket_t noteOff = {0x08, 0x80 | channel - 1, pitch, velocity};
  MidiUSB.sendMIDI(noteOff);
}

void controlChange(byte channel, byte control, byte value) {
  midiEventPacket_t event = {0x0B, 0xB0 | channel - 1, control, value};
  MidiUSB.sendMIDI(event);
}
void programChange(byte channel, byte patch) {
  midiEventPacket_t event = {0x0C, 0xC0 | channel - 1, patch};
  MidiUSB.sendMIDI(event);
}
